<?php

namespace Drupal\search_api_clir\EventSubscriber;

use Drupal\Core\Language\LanguageManagerInterface;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Drupal\Core\StringTranslation\TranslatableMarkup;
use Drupal\locale\StringStorageInterface;
use Drupal\search_api\Plugin\search_api\data_type\value\TextValueInterface;
use Drupal\search_api_solr\Event\PreAddLanguageFallbackFieldEvent;
use Drupal\search_api_solr\Event\SearchApiSolrEvents;
use Symfony\Component\EventDispatcher\EventSubscriberInterface;

/**
 * Search API Solr events subscriber.
 */
class SearchApiSolrSubscriber implements EventSubscriberInterface {

  use StringTranslationTrait;

  /**
   * The string storage for reading and writing translations.
   *
   * @var \Drupal\locale\StringStorageInterface
   */
  protected $localeStorage;

  /**
   * The language manager.
   *
   * @var \Drupal\Core\Language\LanguageManagerInterface
   */
  protected $languageManager;

  /**
   * Creates a new typed configuration manager.
   *
   * @param \Drupal\locale\StringStorageInterface $locale_storage
   *   The locale storage to use for reading string translations.
   */
  public function __construct(StringStorageInterface $locale_storage, LanguageManagerInterface $language_manager) {
    $this->localeStorage = $locale_storage;
    $this->languageManager = $language_manager;
  }

  /**
   * Adds the mapping to treat some Solr special fields as fulltext in views.
   *
   * @param \Drupal\search_api_solr\Event\PreAddLanguageFallbackFieldEvent $event
   *   The Search API Solr event.
   *
   * @throws \Drupal\Core\Entity\EntityStorageException
   */
  public function preAddLanguageFallbackField(PreAddLanguageFallbackFieldEvent $event): void {
    switch ($event->getType()) {
      case 'solr_text_custom:string':
        // Don't translate "strings".

        break;

      case 'text':
      default:
        $item = $event->getItem();
        if ($item->getLanguage() === $this->languageManager->getDefaultLanguage()->getId()) {
          $langcode = $event->getLangcode();
          $value = $event->getValue();
          if (!is_array($value)) {
            $value = [$value];
          }

          foreach ($value as $text_value) {
            if ($text_value instanceof TextValueInterface) {
              $text = $text_value->getText();
              // Previous processing steps like 'auto_aggregated_fulltext_field'
              // might already have triggered a translation.
              if (!($text instanceof TranslatableMarkup)) {
                if ($translation = $this->localeStorage->findString([
                  'source' => $text,
                  'language' => $langcode,
                  'translated' => TRUE
                ])) {
                  // There's an existing translation from any context. We simply
                  // use it.
                  $text_value->setText($this->t($text, [], [
                    'langcode' => $langcode,
                    'context' => $translation->context,
                  ]));
                }
                else {
                  // The context identifier has a maximum length of 255
                  // characters. The item ID is limited to 150 characters, a UUID
                  // has 36 characters. So the concatination should be safe.
                  $context = 'search_api_clir;' . $event->getIndex()
                      ->uuid() . '|' . $item->getId();
                  $translations = $this->localeStorage->getStrings([
                    'source' => $text,
                    'language' => $langcode,
                    'translated' => FALSE
                  ]);
                  foreach ($translations as $translation) {
                    if ($translation->context && strpos($translation->context, 'search_api_clir;') === 0) {
                      // @todo If the translation is stored in a different
                      //   search_api_clir context, store the current context
                      //   somewhere else as related. That will allow to
                      //   trigger re-indexing for the different item as well
                      //   when the translation arrives.
                      continue 2;
                    }
                  }

                  // The text does not exist, or it exists in a different context
                  // without a translation. Register it within the
                  // search_api_clear context.
                  $translatable_markup = $this->t($text, [], [
                    'langcode' => $langcode,
                    'context' => $context,
                  ]);
                  $text_value->setText($translatable_markup);
                }
              }
            }
          }

          $event->setValue($value);
        }

        break;
    }
  }

  /**
   * {@inheritdoc}
   */
  public static function getSubscribedEvents(): array {
    return [
      SearchApiSolrEvents::PRE_ADD_LANGUAGE_FALLBACK_FIELD => 'preAddLanguageFallbackField',
    ];

  }

}
